
#ifndef __ISCENEOBJECT_H_
#define __ISCENEOBJECT_H_
/*
Peon - Win32 Games Programming Library
Copyright (C) 2002-2005 Erik Yuzwa

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Erik Yuzwa
peon AT wazooinc DOT com
*/

#include "IUnknown.h"
#include "Vector3.h"

namespace peon
{
	/**
	* This base object for our scene 
	*/
	class PEONMAIN_API ISceneObject : public IUnknown
	{
	public:
		/** position in the game world */
		Vector3		m_vecPos;

	public:
		/**
		* Constructor
		*/
		ISceneObject(){}

		/**
		* Destructor
		*/
		virtual ~ISceneObject(){}


		/**
		* This method is used to update this particular node.
		* Every object inherited from ISceneObject should try
		* to override this method, as it allows it to be
		* processed by any scene manager.
		* @param fElapsedTime - elapsed time between frames
		* @return void
		*/
		virtual void onUpdate(float fElapsedTime){}

		/**
		* This method can be used as a overrideable for
		* rendering
		* @return void
		*/
		virtual void onRender(){}

		/**
		* This method just sets our object's position
		* @param newPos - Vector3 of the game world
		* @return void
		*/
		void setPos( Vector3 newPos ){ m_vecPos = newPos; }

	};
}

#endif