
#ifndef __INPUTENGINE_H_
#define __INPUTENGINE_H_
/*
Peon - Win32 Games Programming Library
Copyright (C) 2002-2005 Erik Yuzwa

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Erik Yuzwa
peon AT wazooinc DOT com
*/

#include "ISingleton.h"

#define JOYSTICK_DEAD_ZONE 32768.0f


namespace peon
{
	/**
	* This object keeps an eye on the input received by SDL
	*
	* This singleton object is a manager "of sorts" for the input 
	* devices available on the system. 
	*/
	class PEONMAIN_API InputEngine : public ISingleton<InputEngine>
	{
	protected:
		/** handle to the joystick */
		SDL_Joystick*		m_pJoystick;

		/** is there a joystick connected? */
		bool			    m_bJoystickConnected;

	public:
		/**
		* Constructor
		*/
		InputEngine();

		/**
		* Destructor
		*/
		~InputEngine();

		/** Override standard Singleton retrieval.
		@remarks
		Why do we do this? Well, it's because the Singleton
		implementation is in a .h file, which means it gets compiled
		into anybody who includes it. This is needed for the
		Singleton template to work, but we actually only want it
		compiled into the implementation of the class based on the
		Singleton, not all of them. If we don't change this, we get
		link errors when trying to use the Singleton-based class from
		an outside dll.
		@par
		This method just delegates to the template version anyway,
		but the implementation stays in this single compilation unit,
		preventing link errors.
		*/
		static InputEngine& getSingleton(void);
		/** Override standard Singleton retrieval.
		@remarks
		Why do we do this? Well, it's because the Singleton
		implementation is in a .h file, which means it gets compiled
		into anybody who includes it. This is needed for the
		Singleton template to work, but we actually only want it
		compiled into the implementation of the class based on the
		Singleton, not all of them. If we don't change this, we get
		link errors when trying to use the Singleton-based class from
		an outside dll.
		@par
		This method just delegates to the template version anyway,
		but the implementation stays in this single compilation unit,
		preventing link errors.
		*/
		static InputEngine* getSingletonPtr(void);


		/**
		* This method loads up and configures our input devices. SDL 
		* will automagically create our handles to the keyboard and
		* mouse device, so we're really only talking about initializing
		* a handle to the joystick.
		*
		* I always return true, as I feel the game should not bail just
		* because the joystick was not found/inited properly. Feel free
		* to modify depending upon your game's needs.
		*
		* @param pConfig - a valid IniConfigReader instance
		* @return bool - always true
		*/
		bool loadEngine( IniConfigReader* pConfig );

		/**
		* This method just unloads and frees any allocated input devices
		*/
		void unloadEngine();

		/**
		* This method returns the current X axis of the joystick
		* @return Sint16 - value of our x axis
		*/
		Sint16 getJoyXAxis();

		/**
		* This method returns the current Y axis of the joystick
		* @return Sint16 - value of our y axis
		*/
		Sint16 getJoyYAxis();

	};
}

#endif
