
#ifndef __PRIMTYPES_H_
#define __PRIMTYPES_H_
/*
Peon - Win32 Games Programming Library
Copyright (C) 2002-2005 Erik Yuzwa

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Erik Yuzwa
peon AT wazooinc DOT com
*/

#include "Peonstdafx.h"

namespace peon
{

	/**
	* This scene primitive type is our basic starting point for getting
	* geometry rendered to our screen. 
	*/
	struct PEONMAIN_API DiffusePrim
	{
		/** x, y, z position of this vertex */
		float x, y, z;

		/** diffuse color of this vertex */
		float r, g, b, a;
	};

	/**
	* This scene primitive is a starting point for displaying geometry
	* that can contain texture coordinates.
	*/
	struct PEONMAIN_API DiffuseTexPrim
	{
		/** x, y and z position of this vertex */
		float x, y, z;

		/** diffuse color component of this vertex */
		float r, g, b, a;

		/** texture coordinates of this vertex */
		float tu, tv;
	};

	/**
	* This scene primitive is a starting point for displaying any type
	* of geometry in our game world. It contains enough information to
	* properly help lighting, blending and texture mapping calculations.
	*/
	struct PEONMAIN_API NormalDiffuseTexPrim
	{
		/** x, y and z position of this vertex */
		float x, y, z;

		/** x, y and z components of the vector normal */
		float nx, ny, nz;

		/** diffuse color component of this vertex */
		float r, g, b, a;

		/** texture coordinates of this vertex */
		float tu, tv;

	};
}

#endif

