
#ifndef __SCENEROOT_H_
#define __SCENEROOT_H_
/*
Peon - Win32 Games Programming Library
Copyright (C) 2002-2005 Erik Yuzwa

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Erik Yuzwa
peon AT wazooinc DOT com
*/


#include "ISingleton.h"
#include "SceneRenderer.h"
#include "ISceneObject.h"

namespace peon
{
	/**
	* This object represents the "Base node" of a scene graph for
	* you to play with. Right now this is little more than a display
	* list, but it gives you an idea of what's possible. To create
	* a true Octree or QuadTree structure is left as your own
	* exercise.
	*/
	class PEONMAIN_API SceneRoot : public ISceneObject, ISingleton<SceneRoot>
	{
	protected:
		/** our SceneRenderer handle */
		SceneRenderer*	m_pRenderer;

	public:

		/**
		* Constructor
		*/
		SceneRoot( SceneRenderer* pRenderer );

		/**
		* Destructor
		*/
		~SceneRoot();

		/** Override standard Singleton retrieval.
		@remarks
		Why do we do this? Well, it's because the Singleton
		implementation is in a .h file, which means it gets compiled
		into anybody who includes it. This is needed for the
		Singleton template to work, but we actually only want it
		compiled into the implementation of the class based on the
		Singleton, not all of them. If we don't change this, we get
		link errors when trying to use the Singleton-based class from
		an outside dll.
		@par
		This method just delegates to the template version anyway,
		but the implementation stays in this single compilation unit,
		preventing link errors.
		*/
		static SceneRoot& getSingleton(void);

		/** Override standard Singleton retrieval.
		@remarks
		Why do we do this? Well, it's because the Singleton
		implementation is in a .h file, which means it gets compiled
		into anybody who includes it. This is needed for the
		Singleton template to work, but we actually only want it
		compiled into the implementation of the class based on the
		Singleton, not all of them. If we don't change this, we get
		link errors when trying to use the Singleton-based class from
		an outside dll.
		@par
		This method just delegates to the template version anyway,
		but the implementation stays in this single compilation unit,
		preventing link errors.
		*/
		static SceneRoot* getSingletonPtr(void);


	};
}

#endif
